#pragma once

#include"offline.h"
#include"../manager_constants.h"
#include<vector>
#include<algorithm>
#include<unordered_set>
#include<random>
#include<cassert>

using namespace std;
typedef __int128 bitstring;

class collisions_adversary : public offline_adversary
{
 public:
    collisions_adversary(FILE* fin) : outcomes(N + 1, 0)
    {
        int s;
        assert(fscanf(fin, "%d", &s) == 1);
        rnd_engine.seed(s);
    }

    bool answer(int x, int y)
    {
        if(x < y) swap(x, y);
    
        for(int a = 1; a <= N; ++a)
        {
            for(int b = 1; b <=a; ++b)
            {
                bitstring curr_outcome = outcomes[a] | outcomes[b];
                if(curr_outcome == actual_outcome and (a != x or b != y))
                    return false;
            }
        }
        
        return true;
    }
    
 protected:
    vector<int> get_replies(vector<vector<int>> queries)
    {
        bitstring mask = 1;
        
        for(int i = 0; i < queries.size(); ++i)
        {
            for(int x : queries[i])
                outcomes[x] += mask;
            mask <<= 1;
        }

        unordered_set<bitstring> results;

        for(int x = 1; x <= N; ++x)
        {
            for(int y = 1; y <= x; ++y)
            {
                bitstring curr_outcome = outcomes[x] | outcomes[y];
            
                if(results.find(curr_outcome) == results.end())
                {
                    results.insert(curr_outcome);
                }
                
                else
                {
                    #ifdef DEBUG
                        fprintf(stderr, "collision: %d, %d\n", x, y);
                    #endif
                
                    // We could just decide to not accept the answer at this
                    // point -- however, we will instead actually find a
                    // different possible set of locations in answer (this might
                    // be helpful for debugging purposes)
                    actual_outcome = curr_outcome;
                    return reply_from_bitstring(actual_outcome, queries.size());
                }
            }
        }

        uniform_int_distribution<int> dis(1, 1000);        
        actual_outcome = outcomes[dis(rnd_engine)] | outcomes[dis(rnd_engine)];
        return reply_from_bitstring(actual_outcome, queries.size());
    }

 private:
    vector<bitstring> outcomes;
    mt19937 rnd_engine;
    bitstring actual_outcome;
    
    vector<int> reply_from_bitstring(bitstring s, int length)
    {
        vector<int> reply;
        bitstring mask = 1;
    
        for(int i = 0; i < length; ++i)
        {
            reply.push_back(s & mask ? 1 : 0);
            mask <<= 1;
        }
        
        return reply;
    }

};
