/* An H=2 strategy that prefers either many successful returns or many
 * unsuccessful returns
 *
 * Running time: O(N²)
 * Memory:       O(N²)
 */

#pragma once

#include"two_phase.h"
#include<cassert>

class preference_adversary : public two_phase_adversary
{
 public:
    preference_adversary(int preference, int s) : two_phase_adversary(s), preference(preference)
    {}

 protected:
    bitstring first_reply(const block_map& block_size, vector<vector<int>> _)
    {
        int best = -1;    
        for(auto b : block_size) best = max(best, value(b.first));

        vector<int> possible_outcomes;
        for(auto b : block_size)
            if(value(b.first) == best)
                possible_outcomes.push_back(b.first);

        assert(not possible_outcomes.empty());

        uniform_int_distribution<int> distr(0, possible_outcomes.size() - 1);
        bitstring curr_outcome = possible_outcomes[distr(rnd_engine)];

        #ifdef DEBUG
            fprintf(stderr, "[outcome %x for first run]", curr_outcome);
        #endif

        return curr_outcome;
    }

 private:
    int preference;

    int value(bitstring b)
    {
        bitstring mask = 1;
        int r = 0;

        while(mask)
        {
            if((bool)(b & mask) == (bool)(preference))
                ++r;
            mask <<= 1;
        }
        
        return r;
    }
};
