/* An H=2 strategy that takes a random consistent outcome in the first step
 *
 * Running time: O(N²)
 * Memory:       O(N²)
 */

#pragma once

#include"two_phase.h"
#include<cassert>
#include<random>

class rnd_adversary : public two_phase_adversary
{
 public:
    rnd_adversary(int s) : two_phase_adversary(s)
    {}

 protected:
    bitstring first_reply(const block_map& block_size, vector<vector<int>> _)
    {
        vector<bitstring> possible_outcomes;
    
        for(auto b : block_size)
            possible_outcomes.push_back(b.first);

        assert(not possible_outcomes.empty());

        uniform_int_distribution<int> distr(0, possible_outcomes.size() - 1);
        bitstring curr_outcome = possible_outcomes[distr(rnd_engine)];

        #ifdef DEBUG
            fprintf(stderr, "[outcome %x for first run]", curr_outcome);
        #endif

        return curr_outcome;
    }
};
