/* An H=2 strategy that prefers the outcomes for the individual scouts to be
 * either vastly different or very close to each other
 *
 * Running time: O(N²)
 * Memory:       O(N²)
 */

#pragma once

#include"../manager_constants.h"
#include"two_phase.h"
#include<cassert>

class xor_adversary : public two_phase_adversary
{
 public:
    xor_adversary(int preference, int s) : two_phase_adversary(s), preference(preference)
    {}

 protected:
    bitstring first_reply(const block_map& block_size, vector<vector<int>> q)
    {
        int l = q.size();
    
        unordered_map<bitstring, long long> total_value;
        long long best = -1;
        
        for(int x = 1; x <= N; ++x)
        {        
            for(int y = 1; y <= x; ++y)
            {
                bitstring curr_outcome = outcomes[x] | outcomes[y];
                                
                total_value[curr_outcome] += value(outcomes[x], outcomes[y], l);
                best = max(best, total_value[curr_outcome]);
            }
        }

        #ifdef DEBUG
            fprintf(stderr, "[best total_value is %lld]", best);
        #endif

        vector<bitstring> possible_outcomes;
        for(auto b : total_value)
            if(b.second == best) possible_outcomes.push_back(b.first);

        assert(not possible_outcomes.empty());

        uniform_int_distribution<int> distr(0, possible_outcomes.size() - 1);
        bitstring curr_outcome = possible_outcomes[distr(rnd_engine)];

        #ifdef DEBUG
            fprintf(stderr, "[outcome %x for first run]", curr_outcome);
        #endif

        return curr_outcome;
    }

 private:
    int preference;

    long long value(bitstring a, bitstring b, int size)
    {
        bitstring delta = a ^ b;
        bitstring mask = 1;
        long long r = 0;

        for(int i = 0; i < size; ++i)
        {
            if((bool)(delta & mask) == (bool)(preference))
                ++r;
            mask <<= 1;
        }

        return r * r; // we square this to skew in favour of extreme values
    }
};
