// #define DEBUG

#include<vector>
#include<string>
#include<algorithm>
#include<cstdio>
#include<cassert>
#include"mystubmanagerframework.h"
#include"manager_constants.h"
#include"adversaries/lib.h"
#include"scoreinfo.h"

using namespace std;

constexpr int SEND = 114738923, WAIT = 752019792, ANSWER = 539014118;

#ifdef DEBUG
    #define OUT_OF_RAVENS "Out of ravens"
    #define OUT_OF_TIME "Out of time"
    #define INVALID_SEND "Invalid send"
    #define WRONG_ANSWER(x,y) "Wrong answer %d %d", x, y
    #define CORRECT(r,h) "Correct: %d raven(s) used, %d hour(s) passed", r, h
    #define SECURITY_VIOLATION "Security violation!"
#else
    #define OUT_OF_RAVENS "Not correct"
    #define OUT_OF_TIME "Not correct"
    #define INVALID_SEND "Not correct"
    #define WRONG_ANSWER(x,y) "Not correct"
    #define CORRECT(r,h) "Correct: %d robot(s) used, %d hour(s) passed", r, h
    #define SECURITY_VIOLATION "Security violation!"
#endif

int R, H;
int r = 0, h = 0;
int use_partial_scoring;

adversary* adv;

template<typename T> void write(T t) {
  fwrite(&t, sizeof(t), 1, fcommout);
  fflush(fcommout);
}

template<typename T> T read_or_fail() {
  T x;
  if(fread(&x, sizeof(x), 1, fcommin) != 1)
    result(0.0, SECURITY_VIOLATION);
  return x;
}

void handle_send() {
    ++r;
    if(r > R) result(0.0, OUT_OF_RAVENS);
    write(1); // This is fine

    int k = read_or_fail<int>();
    if(k < 0 || k > N) result(0.0, INVALID_SEND);

    vector<short> shortdests(k);
    if(fread(shortdests.data(), sizeof(short), k, fcommin) != (size_t) k)
        result(0.0, SECURITY_VIOLATION);
    vector<char> is_dest(N + 1, false);

    vector<int> dests; dests.reserve(k);
    for(int x : shortdests) {
        if(x <= 0 or x > N or is_dest[x])
            result(0.0, INVALID_SEND);
        is_dest[x] = true;
        dests.push_back(x);
    }

    adv->send(dests);

    // Tell the user program that everything is fine
    write(1);
}

void handle_wait() {
    ++h;
    if(h > H) result(0.0, OUT_OF_TIME);

    vector<int> answers = adv->wait();

    write(static_cast<int>(answers.size()));
    if(fwrite(answers.data(), sizeof(int), answers.size(), fcommout) != answers.size())
        result(0.0, SECURITY_VIOLATION);
    fflush(fcommout);
}

void handle_answer() {
    int x = read_or_fail<int>(), y = read_or_fail<int>();

    #ifdef DEBUG
        fprintf(stderr, "(user answers %d,%d)", x, y);
    #endif

    if(1 <= x and x <= N and 1 <= y and y <= N and adv->answer(x, y)) {
        char buffer[1024]; sprintf(buffer, CORRECT(r,h));
        result(vector<float>({1.0, (float) score[r]}), vector<string>({buffer, buffer}));
    }
    else
        result(0.0, WRONG_ANSWER(x, y));
}

void check() {
    message_on_shutdown = true;

    assert(fscanf(fin, "%d %d", &R, &H) == 2);
    write(R); write(H);

    adv = create_adversary(fin);

    for(;;) {
        int type = read_or_fail<int>();

        switch(type) {
            case SEND:
                handle_send();
                break;

            case WAIT:
                handle_wait();
                break;

            case ANSWER:
                handle_answer();
                break;

            default:
                result(0.0, SECURITY_VIOLATION);
                break;
        }
    }
}
